from pres.combined.diagrams import *
from math import *
from myfonts import fonts

line_thickness = 0.05

def radians( angle ):
    return angle*(pi/180.0)

def draw_spokes( bone_radius, flesh_radius, bone_angle, flesh_angle, segments ):
    if segments>0:
        da = 360.0/segments
        for i in range(segments):
            angle = i*da
            ix = bone_radius*cos(radians(angle+bone_angle))
            iy = bone_radius*sin(radians(angle+bone_angle))
            ox = flesh_radius*cos(radians(angle+flesh_angle))
            oy = flesh_radius*sin(radians(angle+flesh_angle))
            line( (ix,iy), (ox,oy) )
            line( (ix,iy), (0,0) )

def draw_rings( bone_radius, flesh_radius, rings ):
    if rings>0:
        dr = (flesh_radius-bone_radius)/rings
        r = bone_radius
        for i in range(rings):
            circle( r )
            r += dr
    
def draw_bone( bone_radius ):
    color(yellow)
    dot( bone_radius )
    color(black)
    widestroke( Path().arc( 0, 0, 0, 360, bone_radius ), line_thickness )

def draw_b():
    text( (-0.02,0.0), "B", fonts["italic"], 0.3, 0.0 )

def draw_pe_region( radius, dots ):
    color(black)
    dot_radius = radius*0.07
    if dots>0:
        da = 360.0/dots
        for i in range(dots):
            angle = i*da
            x = radius*cos(radians(angle))
            y = radius*sin(radians(angle))
            push()
            translate(x,y)
            dot( dot_radius )
            pop()
    
def bone( show_bone=(BOOLEAN,1),
          pe_region=(SCALAR,0.0,1.0,0.0),
          pe_radius=(SCALAR,0.0,1.0,0.25),
          outer_angle=(INTEGER,-90,90,0),
          inner_angle=(INTEGER,-90,90,0),
          bone_radius=(SCALAR,0.0,1.0,0.25),
          segments=(INTEGER,3,16,8),
          rings=(INTEGER,1,4,2),
          show_b=(SCALAR,0.0,1.0,0.0) ):        
    flesh_radius = 0.75
    pe_color_scalar = 0.07
    skel_radius = 0.1
    clear(white)

    if pe_region>0.0:
        pe_alpha = fabs(outer_angle)*pe_color_scalar*0.25
    else:
        pe_alpha = fabs(outer_angle)*pe_color_scalar*bone_radius

    color(red,pe_alpha)
    dot( flesh_radius )

    color(black)
    widestroke( Path().arc( 0, 0, 0, 360, flesh_radius ), line_thickness )    

    draw_spokes( bone_radius, flesh_radius, inner_angle, outer_angle, segments )    
    draw_rings( bone_radius, flesh_radius, rings )

    if pe_region>0.0:
        color( Color(0.8,0.8,0.0), pe_region )
        draw_pe_region( pe_radius, 29 )

    if show_b>0.0:        
        color( black, show_b )
        draw_b()

    if show_bone:
        draw_bone( bone_radius )

test_diagrams( bone )

                
