from pres.combined.diagrams import *
from math import *
from spline import *

imagepath.append( 'images' )

from myfonts import fonts

rest_points = [ (-0.5,0.75), (0.0,0.75), (0.5,0.75), (0.5,0.0),
                (0.5,-0.75), (0.0,-0.75), (-0.5,-0.75), (-0.5,0.0) ]

deformation = [ (-1.0,1.0), (0.0,-0.5), (1.0,1.0), (-0.5,0.0),
                (1.0,-1.0), (0.0,0.5), (-1.0,-1.0), (0.5,0.0) ]

def add_points( p, q, scale=1.0 ):
    rng = range(len(p))
    result = []
    for i in rng:
        result.append( (p[i][0]+(scale*q[i][0]),p[i][1]+(scale*q[i][1])) )
    return result

def rotate_points( p, angle ):
    sina = sin(angle)
    cosa = cos(angle)
    result = []
    for pt in p:
        result.append( (pt[0]*cosa+pt[1]*sina, -pt[0]*sina+pt[1]*cosa) )
    return result

def translate_points( p, x ):
    result = []
    for pt in p:
        result.append( (pt[0]+x,pt[1]) )
    return result

def old_build_path( p ):
    path = Path()
    path.moveto(p[0][0],p[0][1])
    path.qcurveto(p[1][0],p[1][1],p[2][0],p[2][1])
    path.qcurveto(p[3][0],p[3][1],p[4][0],p[4][1])
    path.qcurveto(p[5][0],p[5][1],p[6][0],p[6][1])
    path.qcurveto(p[7][0],p[7][1],p[0][0],p[0][1])
    path.closepath()
    return path;

rest_path = old_build_path( rest_points )

def build_path( p ):
    path = Path()
    catmullrom( path, (p[0][0],p[0][1]),(p[1][0],p[1][1]),(p[2][0],p[2][1]) )
    catmullrom( path, (p[2][0],p[2][1]),(p[3][0],p[3][1]),(p[4][0],p[4][1]) )
    catmullrom( path, (p[4][0],p[4][1]),(p[5][0],p[5][1]),(p[6][0],p[6][1]) )
    catmullrom( path, (p[6][0],p[6][1]),(p[7][0],p[7][1]),(p[0][0],p[0][1]) )
    return path

def draw_arrows( p, q ):
    rng = range(len(p))
    result = []
    for i in rng:
        path = Path().moveto( p[i][0], p[i][1] ).lineto( q[i][0], q[i][1] )
        widestroke(path, 0.03)
        arrow( path, (0,0.1,0.1) )

def draw_plus():
    rectangle( (-1,-0.15), (1,0.15) )
    rectangle( (-0.15,-1), (0.15,1) )

def draw_equals():
    rectangle( (-1,0.35), (1,0.65) )
    rectangle( (-1,-0.35), (1,-0.65) )

rpath = Path().moveto(-1.5,0).qcurveto( 0,0.3,1,0)
def draw_rotate():
    widestroke(rpath, 0.3)
    arrow( rpath, (0,1.0,1.0) )

def draw_bone():
    w = 0.05
    h = 0.4
    radius = 2*w
    rectangle( (-w,-h), (w,h) )
    dot( radius, (0,h) )
    dot( radius, (0,-h) )

highlight_circle = Path().arc( 0, 0, 0, 360, 1.3 ).closepath()

def block( slide_right=(SCALAR,0.0,1.0,0.0),
           show_displacement = (SCALAR,0.0,1.0,1.0),
           circle_displacement = (SCALAR, 0.0, 1.0, 0.0 ),
           deform = ( SCALAR, -0.3, 0.3, 0.0 ),
           rest_angle = ( SCALAR, -1.0, 1.0, 0.0 ),
           final_angle = (SCALAR, -1.0, 1.0, 0.0 ),
           gap = ( SCALAR, 0.0, 1.0, 0.0 ),
           use_rotated = ( SCALAR, 0.0, 1.0, 1.0 ),
           show_bone = ( SCALAR, 0.0, 1.0, 1.0 )):
    clear(white)
    scale(0.25,0.25)
    deformed_points = rotate_points( add_points( rest_points,
                                                 deformation,
                                                 deform ), final_angle)
    rotated_points = rotate_points( rest_points, rest_angle )

    rotated_path = old_build_path( rotated_points )
    
    if use_rotated:
        gap = 1
        
    push()
    sep = 1.5
    stroke_width = 0.05

    color(black)
    translate( (slide_right-2-gap)*sep, 0.0 )
    widestroke( rest_path, stroke_width )
    text( (0.0, -1.5), "rest state", fonts["text"], 0.4, 0.0 )

    if show_bone:
        color(red,show_bone)
        draw_bone()
        color(black,1)
    
    if use_rotated:
        translate( sep, 0.0 )
        push()
        scale(0.3)
        color(black,use_rotated)
        draw_rotate()
        pop()
        color(black,use_rotated*(1.0-show_bone))
        text( (0.0, 0.4), "R", fonts["text"], 0.4, 0.0 )
        color(black,use_rotated)
        translate( sep, 0.0 )
        widestroke( rotated_path, stroke_width )
        text( (0.0, -1.5), " rotated\nrest state", fonts["text"], 0.4, 0.0 )
        if show_bone:
            color(red,use_rotated*show_bone)
            push()
            rotate( -(180.0/pi)*rest_angle )
            draw_bone()
            pop()
            color(black,use_rotated*show_bone)
            translate( -sep, 0.0 )
            text( (0.0, 0.4), "R(bone)", fonts["text"], 0.4, 0.0 )
            translate( sep, 0.0 )

        color(black,1)   
        translate( sep, 0.0 )
    else:
        translate( (2*gap+1)*sep, 0.0 )

    if show_displacement>0.0:        
        push()
        scale(0.3)
        color(black,show_displacement)
        draw_plus()
        pop()
        
        translate( sep, 0.0 )

        color( 0.75, 0.75, 0.75, show_displacement )
        if use_rotated:
            fill( rotated_path )
        else:
            fill( rest_path )

        color(black, show_displacement)
        
        if use_rotated:
            draw_arrows( rotated_points, deformed_points )
        else:
            draw_arrows( rest_points, deformed_points )
            
        text( (0.0, -1.5), "displacement", fonts["text"], 0.4, 0.0 )

        if circle_displacement > 0.0:
            color(red,circle_displacement*show_displacement)
            widestroke( highlight_circle, 0.1 )
            color(black,show_displacement)
            
        translate( sep, 0.0 )
        push()
        scale(0.3)
        draw_equals()
        pop()
    else:
        translate( 2*sep, 0.0 )
    
    translate( sep, 0.0 )
#    color( 0.5, 0.5, 0.5 )
#    fill( build_path(deformed_points) )
    color(black)
    widestroke( build_path(deformed_points), stroke_width )
    text( (0.0, -1.5), "deformed state", fonts["text"], 0.4, 0.0 )

    if show_bone:
        color(red,show_bone)
        push()
        rotate( -(180.0/pi)*rest_angle )
        draw_bone()
        pop()
        color(black,1)

    pop()
    
def block2( tuck = (SCALAR, 0.0, 1.0, 0.0 ),
            deform = ( SCALAR, -0.3, 0.3, 0.0 ),
            rest_angle = ( SCALAR, -1.0, 1.0, 0.0 ),
            final_angle = (SCALAR, -1.0, 1.0, 0.0 ),
            show_bone = ( SCALAR, 0.0, 1.0, 0.0 ),
            zoom=(SCALAR,0.1,10.0,1.0),
            show_displacement=(SCALAR,0.0,1.0,0.0)):
    clear(white)
    scale(zoom)
    deformed_points = rotate_points( add_points( rest_points,
                                                 deformation,
                                                 deform ), final_angle)
    rotated_points = rotate_points( rest_points, rest_angle )

    rotated_path = old_build_path( rotated_points )
            
    push()
    sep = 1.5
    stroke_width = 0.05
    tuck_amount = 1.4

    translate( -2*sep, 0.0 )

    translate( tuck*tuck_amount, 0.0 )    
    widestroke( rotated_path, stroke_width )
    text( (0.0, -1.5), "rest state", fonts["text"], 0.4, 0.0 )
    
    if show_bone:
        color(red,show_bone)
        push()
        rotate( -(180.0/pi)*rest_angle )
        draw_bone()
        pop()

    translate( -tuck*tuck_amount, 0.0 )    

    color(black,1)   
    translate( sep, 0.0 )

    push()
    scale(0.3)
    color(black,show_displacement)
    draw_plus()
    pop()
    
    translate( sep, 0.0 )

    color( 0.75, 0.75, 0.75, show_displacement )
    fill( rotated_path )

    color(black, show_displacement)
    draw_arrows( rotated_points, deformed_points )
        
    text( (0.0, -1.5), "displacement", fonts["text"], 0.4, 0.0 )
        
    translate( sep, 0.0 )
    push()
    scale(0.3)
    draw_equals()
    pop()
    
    translate( sep, 0.0 )
#    color( 0.5, 0.5, 0.5 )
#    fill( build_path(deformed_points) )
    translate( -tuck*tuck_amount, 0.0 )    
    color(black)
    widestroke( build_path(deformed_points), stroke_width )
    text( (0.0, -1.5), "deformed state", fonts["text"], 0.4, 0.0 )

    if show_bone:
        color(red,show_bone)
        push()
        rotate( -(180.0/pi)*rest_angle )
        draw_bone()
        pop()
        color(black,1)

    pop()    

def blocks(show_displacement = (SCALAR, 0.0, 1.0, 0.0 ),
           circle_displacement = (SCALAR, 0.0, 1.0, 0.0 ),
           deform = ( SCALAR, -0.3, 0.3, 0.0 ),
           angle = ( SCALAR, -1.0, 1.0, 0.0 ),
           gap = ( SCALAR, 0.0, 1.0, 0.0 ),
           zoom = ( SCALAR, 0.1, 10.0, 1.0 ),
           use_rotated = ( SCALAR, 0.0, 1.0, 0.0 ),
           show_bone = ( SCALAR, 0.0, 1.0, 0.0 ),
           vgap = ( SCALAR, 0.0, 1.0, 0.0 ) ):
    VGAP = 1.7
    scale(zoom)
    translate( 0, 0.2 )
    clear(white)
    if vgap > 0.0:
        translate( 0, VGAP*vgap )
        block( gap, show_displacement, circle_displacement, deform, angle, angle, 0.0, 0.0, 0.0 )
        translate( 0, -VGAP*vgap )
        
    translate( 0, -VGAP*vgap )
    block( 0.0, show_displacement, circle_displacement, deform, angle, angle, gap, use_rotated, show_bone )
    translate( 0, VGAP*vgap )
    
test_diagrams(block2)
